module integration_mod

use param
use dal
use errorHandling
 



public    
interface DoubleIntegration
    module procedure DoubleIntegration1, DoubleIntegration2
   

end interface

interface QGauss
    module procedure QGauss1, QGauss2
   
end interface


real(double) :: xSave, ylSave, yuSave

real(double), private :: xGauss(5) = (/&
            0.1488743389_double, 0.4333953941_double,  &
            0.6794095682_double, 0.8650633666_double, 0.9739065285_double/)

real(double), private :: wGauss(5) = (/0.2955242247_double, &
            0.2692667193_double, 0.2190863625_double, &
            0.1494513491_double, 0.0666713443_double/)
    


integer, private :: nGauss = 2
contains



subroutine InitialiseGauss(n)
    integer, intent(in) :: n

    ! n = 2
    real(double) :: x2(1) = (/0.5773502691/)

    real(double) :: w2(1) = (/1.0_double/)


    ! n = 4
    real(double) :: x4(2) = (/ &
             0.3399810435_double, 0.8611363115_double/)

    real(double) :: w4(2) = (/ &
             0.6521451548_double, 0.3478548451_double/)

    ! n = 6
    real(double) :: x6(3) = (/ &
             0.2386191860_double, 0.6612093864_double, 0.9324695142_double/)

    real(double) :: w6(3) = (/ &
             0.4679139345_double, 0.3607615730_double, 0.1713244923_double/)

    ! n = 10
    real(double) :: x10(5) = (/&
            0.1488743389_double, 0.4333953941_double,  &
            0.6794095682_double, 0.8650633666_double, 0.9739065285_double/)

    real(double) :: w10(5) = (/0.2955242247_double, &
            0.2692667193_double, 0.2190863625_double, &
            0.1494513491_double, 0.0666713443_double/)

    select case(n)
        case(2)
            nGauss = 1
            xGauss(1 : 1) = x2
            wGauss(1 : 1) = w2
    
        case(4)
            nGauss = 2
            xGauss(1 : 2) = x4
            wGauss(1 : 2) = w4
        case(6)
            nGauss = 3
            xGauss(1 : 3) = x6
            wGauss(1 : 3) = w6
        case(10)
            xGauss(1 : 5) = x10
            wGauss(1 : 5) = w10
            nGauss = 5
        case default
            xGauss(1 : 5) = x10
            wGauss(1 : 5) = w10
            nGauss = 5
    end select


end subroutine InitialiseGauss    
!***********************************************************
!
!***********************************************************
real(double) function DoubleIntegration1(xl, xu, yl, yu, IntegrationFunctionx)
    real(double), intent(in) :: xl, xu, yl, yu
  

   
!    interface
!        function IntegrationFunctionX(x)
!            use dal
!            real(double) :: IntegrationFunctionX, x
!        end function IntegrationFunctionX
!    end interface
    real(double) ::  IntegrationFunctionx
 
    external IntegrationFunctionx

!write(*, *) "Double integration ngauss=", nGauss
    ylSave = yl
    yuSave = yu
    DoubleIntegration1 =  QGauss(xl, xu, IntegrationFunctionX)

end function  DoubleIntegration1


!***********************************************************
!
!***********************************************************


real(double) function DoubleIntegration2(xl, xu, yl, yu, parameters, IntegrationFunctionx)
    real(double), intent(in) :: xl, xu, yl, yu
    real(double) :: parameters(:)

   
!    interface
!        function IntegrationFunctionX(x)
!            use dal
!            real(double) :: IntegrationFunctionX, x
!        end function IntegrationFunctionX
!    end interface
    real(double) ::  IntegrationFunctionx
 
    external IntegrationFunctionx

    ylSave = yl
    yuSave = yu
    DoubleIntegration2 =  QGauss(xl, xu, parameters, IntegrationFunctionX)

end function  DoubleIntegration2



!***********************************************************
!
!***********************************************************


real(double) Function QGauss1(a, b, IntegrationFunction)
    real(double), intent(in) :: a, b
   
 
    integer j
    real(double) :: xr, xm, dx, s

    real(double) ::  IntegrationFunction
    external IntegrationFunction   

    if (abs(b - a) < 1e-4) then
        QGauss1 = 0.0_double
        return
    end if

    xm = 0.5_double * (b + a)
    xr = 0.5_double * (b - a)
    s = 0.0_double
    do j = 1, nGauss
        dx = xr * xGauss(j)
       
        s = s + wGauss(j) * (IntegrationFunction(xm + dx) + IntegrationFunction(xm - dx))
      
    end do
    QGauss1 = s * xr 

end function QGauss1




!***********************************************************
!
!***********************************************************


real(double) Function QGauss2(a, b, parameters, IntegrationFunction)
    real(double), intent(in) :: a, b
    real(double) :: parameters(:)   
    
 
    integer j
    real(double) :: xr, xm, dx, s


    real(double) ::  IntegrationFunction
    external IntegrationFunction   

    if (abs(b - a) < 1e-4) then
        QGauss2 = 0.0_double
        return
    end if

    xm = 0.5_double * (b + a)
    xr = 0.5_double * (b - a)
    s = 0.0_double
    do j = 1, nGauss
        dx = xr * xGauss(j)
       
        s = s + wGauss(j) * (IntegrationFunction(xm + dx, parameters) + IntegrationFunction(xm - dx, parameters))
      
    end do
    QGauss2 = s * xr 

end function QGauss2

end module integration_mod





