!******************************************************************
! Module OmdetectAllocation_mod
!
! Contains various subroutines for numerical procedures.
!******************************************************************

module OmAllocation_mod
    use param
    use cal
    use dal
public   
interface AllocateBoolean
    module procedure allocateBoolean1, allocateBoolean2
end interface AllocateBoolean

   
interface Allocate2dBoolean
    module procedure allocate2dBoolean1, allocate2dBoolean2
end interface Allocate2dBoolean




interface AllocateInt8
    module procedure allocateInt81, allocateInt82
end interface AllocateInt8


interface Allocate2dInt8
    module procedure allocate2dInt81, allocate2dInt82
end interface Allocate2dInt8



interface AllocateInt16
    module procedure allocateInt161, allocateInt162
end interface AllocateInt16

interface Allocate2dInt16
    module procedure allocate2dInt161, allocate2dInt162
end interface Allocate2dInt16


interface AllocateInt32
    module procedure allocateInt321, allocateInt322
end interface AllocateInt32

interface Allocate2dInt32
    module procedure allocate2dInt321, allocate2dInt322
end interface Allocate2dInt32


interface AllocateSingle
    module procedure allocateSingle1, allocateSingle2
end interface AllocateSingle

interface Allocate2dSingle
    module procedure allocate2dSingle1, allocate2dSingle2
end interface Allocate2dSingle

interface Allocate3dSingle
    module procedure allocate3dSingle1, allocate3dSingle2
end interface Allocate3dSingle

interface Allocate4dSingle
    module procedure allocate4dSingle1, allocate4dSingle2
end interface Allocate4dSingle

interface AllocateDouble
    module procedure allocateDouble1, allocateDouble2
end interface AllocateDouble


interface Allocate2dDouble
    module procedure allocate2dDouble1, allocate2dDouble2
end interface Allocate2dDouble






 
       
contains

!***************************
! Function AllocateBoolean
!***************************
function AllocateBoolean1(il, iu, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu
    character(*), intent(in) :: name
    logical(boolean), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = .false.
end function AllocateBoolean1
function AllocateBoolean2(il, iu, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu
    character(*), intent(in) :: name
    logical(boolean), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = .false.
end function AllocateBoolean2



!***************************
! Function AllocateInt8
!***************************
function AllocateInt81(il, iu, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu
    character(*), intent(in) :: name
    integer(int8), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function AllocateInt81
!***************************
! Function AllocateInt8
!***************************
function AllocateInt82(il, iu, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu
    character(*), intent(in) :: name
    integer(int8), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function AllocateInt82
!***************************
! Function AllocateInt16
!***************************
function AllocateInt161(il, iu, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu
    character(*), intent(in) :: name
    integer(int16), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function AllocateInt161
function AllocateInt162(il, iu, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu
    character(*), intent(in) :: name
    integer(int16), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function AllocateInt162

!***************************
! Function AllocateInt32
!***************************
function AllocateInt321(il, iu, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu
    character(*), intent(in) :: name
    integer(int32), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function AllocateInt321

!***************************
! Function AllocateInt32
!***************************
function AllocateInt322(il, iu, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu
    character(*), intent(in) :: name
    integer(int32), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function AllocateInt322


!***************************
! Function Allocate2DInt8
!***************************
function Allocate2DBoolean1(il, iu, jl, ju, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    logical(boolean), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = .false.
end function Allocate2DBoolean1

!***************************
! Function Allocate2DInt8
!***************************
function Allocate2DBoolean2(il, iu, jl, ju, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    logical(boolean), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = .false.
end function Allocate2DBoolean2





!***************************
! Function Allocate2DInt8
!***************************
function Allocate2DInt81(il, iu, jl, ju, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    integer(int8), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function Allocate2DInt81

!***************************
! Function Allocate2DInt8
!***************************
function Allocate2DInt82(il, iu, jl, ju, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    integer(int8), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function Allocate2DInt82



!***************************
! Function Allocate2DInt16
!***************************
function Allocate2DInt161(il, iu, jl, ju, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    integer(int16), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function Allocate2DInt161

!***************************
! Function Allocate2DInt16
!***************************
function Allocate2DInt162(il, iu, jl, ju, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    integer(int16), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function Allocate2DInt162




!***************************
! Function Allocate2DInt32
!***************************
function Allocate2DInt321(il, iu, jl, ju, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    integer(int32), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function Allocate2DInt321

!***************************
! Function Allocate2DInt32
!***************************
function Allocate2DInt322(il, iu, jl, ju, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    integer(int32), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0
end function Allocate2DInt322




!***************************
! Function AllocateSingle1
!***************************
function AllocateSingle1(il, iu, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu
    character(*), intent(in) :: name
    real(single), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function AllocateSingle1

!***************************
! Function AllocateSingle2
!***************************
function AllocateSingle2(il, iu, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu
    character(*), intent(in) :: name
    real(single), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function AllocateSingle2





!***************************
! Function Allocate2DSingle
!***************************
function Allocate2DSingle1(il, iu, jl, ju, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    real(single), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function Allocate2DSingle1

!***************************
! Function Allocate2DSingle
!***************************
function Allocate2DSingle2(il, iu, jl, ju, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu, jl, ju
    character(*), intent(in) :: name
    real(single), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function Allocate2DSingle2






!***************************
! Function AllocateDouble
!***************************
function AllocateDouble1(il, iu, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu
    character(*), intent(in) :: name
    real(double), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0_double
end function AllocateDouble1

!***************************
! Function AllocateDouble
!***************************
function AllocateDouble2(il, iu, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu
    character(*), intent(in) :: name
    real(double), dimension(:), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0_double
end function AllocateDouble2





!***************************
! Function Allocate2DDouble
!***************************
function Allocate2DDouble1(il, iu, jl, ju, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu, jl, ju
    character(len=*), intent(in) :: name
    real(double), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0_double
end function Allocate2DDouble1

!***************************
! Function Allocate2DDouble
!***************************
function Allocate2DDouble2(il, iu, jl, ju, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu, jl, ju
    character(len=*), intent(in) :: name
    real(double), dimension(:, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0_double
end function Allocate2DDouble2




!***************************
! Function AllocateChar
!***************************
function AllocateChar(length, size, name) result(arrayPtr)
    integer(int32), intent(in) :: length, size
    character(*), intent(in) :: name
    character(length), pointer, dimension(:) :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(1 : size), stat = status)
    call CheckAllocation(status, name)

end function AllocateChar




!***************************
! Function Allocate3DSingle
!***************************
function Allocate3DSingle1(il, iu, jl, ju, kl, ku, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu, jl, ju, kl, ku
    character(*), intent(in) :: name
    real(single), dimension(:, :, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju, kl : ku), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function Allocate3DSingle1

!***************************
! Function Allocate3DSingle
!***************************
function Allocate3DSingle2(il, iu, jl, ju, kl, ku, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu, jl, ju, kl, ku
    character(*), intent(in) :: name
    real(single), dimension(:, :, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju, kl : ku), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function Allocate3DSingle2




!***************************
! Function Allocate4DSingle
!***************************
function Allocate4DSingle1(il, iu, jl, ju, kl, ku, ll, lu, name) result(arrayPtr)
    integer(int32), intent(in) :: il, iu, jl, ju, kl, ku, ll, lu
    character(*), intent(in) :: name
    real(single), dimension(:, :, :, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju, kl : ku, ll : lu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function Allocate4DSingle1


!***************************
! Function Allocate4DSingle
!***************************
function Allocate4DSingle2(il, iu, jl, ju, kl, ku, ll, lu, name) result(arrayPtr)
    integer(int16), intent(in) :: il, iu, jl, ju, kl, ku, ll, lu
    character(*), intent(in) :: name
    real(single), dimension(:, :, :, :), pointer :: arrayPtr
    integer :: status
    
    allocate(arrayPtr(il : iu, jl : ju, kl : ku, ll : lu), stat = status)
    call CheckAllocation(status, name)
    arrayPtr = 0.0
end function Allocate4DSingle2




!***************************************************
! CheckAllocation
!***************************************************
subroutine CheckAllocation(status, string)
    integer (int32), intent(in) ::  status
    character(*), intent(in) :: string
    character(100) :: comment
   
    if(status .ne. 0) then
       write(comment, '(a32, a20)') "Could not allocate memory for : ", string 
       write (errstr, *) comment
       call fatal("allocErr") 
    end if
   
end subroutine CheckAllocation

!***************************************************
! CheckDeallocation
!***************************************************
subroutine CheckDeallocation(status, string)
    integer (int32), intent(in) ::  status
    character(*), intent(in) :: string
    character(100) :: comment
   
    if(status .ne. 0) then
       write(comment, '(a31, a20)') "Failed to release memory for : ", trim(string) 
       write (errstr, *) comment
       call warning("allocErr", comment) 
    end if
   
end subroutine CheckDeallocation 

end module OmAllocation_mod









